-- ProccWatch  RIFT AddOn
-- Author: Maggo @ RIFTUI

--[[

Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]

-- Vars

local active = false
local activeBuff = {}
local testMode = false
local loaded = false
local offset = {x=0, y=0}
local pos = {x=0, y=0}
local isDragging = false

-- UI Design
local context = UI.CreateContext("HUD");
local window

-- Notice
function begin()
	if not loaded then
		print(" loaded. Type /pw to see a list of available commands")
		setUp()
		loaded = true
	end
end

-- Interface setup
function setUp()
	if not window then window = UI.CreateFrame("Frame", "Window", context) end
	window:SetVisible(false)
	window:SetBackgroundColor(0,0,0,0.5)
	window:SetHeight(135)
	window:SetWidth(110)
	window:SetLayer(0)
	
	if not ProccWatch_Position.x or not ProccWatch_Position.y then
		window:SetPoint("TOPCENTER", UIParent, "TOPCENTER", 0, 200)
		ProccWatch_Position.x = window:GetLeft()
		ProccWatch_Position.y = window:GetTop()
		window:ClearPoint("TOPCENTER")
	else
		window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", tonumber(ProccWatch_Position.x), tonumber(ProccWatch_Position.y))
	end
	
	if not window.bar then window.bar = UI.CreateFrame("Frame", "Bar", window) end
	window.bar:SetBackgroundColor(.5, 1, 0, .5)
	window.bar:SetHeight(window:GetHeight())
	window.bar:SetWidth(window:GetWidth())
	window.bar:SetPoint("BOTTOMCENTER", window, "BOTTOMCENTER")
	window.bar:SetLayer(1)
	
	if not window.icon then window.icon = UI.CreateFrame("Texture", "Icon", window) end
	window.icon:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 5)
	window.icon:SetWidth(100)
	window.icon:SetHeight(100)
	window.icon:SetLayer(2)
	
	if not window.describe then window.description = UI.CreateFrame("Text", "Skill name", window) end
	window.description:SetPoint("BOTTOMCENTER", window, "BOTTOMCENTER")
	window.description:SetFontSize(21)
	window.description:SetLayer(2)
	
	function window.Event:LeftClick()
		if active then
			window:SetVisible(false)
			active = false
		end
	end

	function window.Event:MouseMove()
		if isDragging and offset then
			local mouse = Inspect.Mouse()
			ProccWatch_Position.x = mouse.x + offset.x
			ProccWatch_Position.y = mouse.y + offset.y
			
			window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ProccWatch_Position.x, ProccWatch_Position.y)
		end
	end

	function window.Event:LeftDown()
		if testMode then
			isDragging = true
			
			local mouse = Inspect.Mouse()
			offset.x = window:GetLeft() - mouse.x
			offset.y = window:GetTop() - mouse.y
		end
	end

	function window.Event:LeftUp()
		if testMode then
			isDragging = false
			print("Position set!")
		end
	end
end


-- On SavedVars Load
function varLoaded()
	if not ProccWatch_Skills then ProccWatch_Skills = {} end
	if not ProccWatch_Position then ProccWatch_Position = {} end
end

-- On Update
function update()
	if active then
		local buff = Inspect.Buff.Detail(Inspect.Unit.Lookup(activeBuff.unit), activeBuff.id);
		
		if buff then
			window.bar:SetHeight(buff.remaining / buff.duration*window:GetHeight())
		else
			active = false
			window:SetVisible(false)
		end
	end
end

-- On Buff Add
function buffAdded(unit, buffs) 
	if not testMode then
		for k,v in pairs(buffs) do
			local buff = Inspect.Buff.Detail(Inspect.Unit.Lookup(unit), k);
			
			if buff and tContains(ProccWatch_Skills, buff.name) and isPlayer(buff.caster) then
				activeBuff = buff
				activeBuff.unit = unit
				activeBuff.id = k
				active = true
				showIcon(buff)
			end
		end
	end
end

-- On Buff Remove
function buffRemoved(unit, buffs)
	for k,v in pairs(buffs) do
		if activeBuff.id == k then 
			window:SetVisible(false) 
			active = false
		end
	end
end

-- Show an icon
function showIcon(buff)
	window.icon:SetTexture("Rift", buff.icon)
	window.description:SetText(buff.name)
	window.description:ResizeToText()
	window:SetWidth(window.description:GetWidth()+10)
	window.bar:SetWidth(window:GetWidth())
	window.bar:SetHeight(window:GetHeight())
	window:SetVisible(true)
end

-- Slash Command
function slashRegistered(commandLine)
	local command = commandLine:match("^(%a+)")
	local params = string.gsub(commandLine, "^(%a+)(%s*)", "")
	
	-- Add a new skill to tracking list
	if command == "add" then
		if params then
			if not tContains(ProccWatch_Skills, params) then
				table.insert(ProccWatch_Skills, params)
				print("Buff ",params," added.")
			else
				print("Buff ",params," tracking already.")
			end
		else
			print("Please add a Buff in your language")
		end
	
	-- remove a skill from tracking
	elseif command == "del" then
		if params then
			if tContains(ProccWatch_Skills, params) then
				table.remove(ProccWatch_Skills, tContains(ProccWatch_Skills, params))
				print("Buff ",params," removed")
			else
				print("Buff ",params," not found")
			end
		else
			print("Please specify a buff to remove")
		end
	-- show a list of tracked skills
	elseif command == "show" then
		print("Saved Skills")
		for k,v in pairs(ProccWatch_Skills) do
			print("-- ",v)
		end
	-- Show test icon
	elseif command == "test" then
		if not testMode then
			print("Testmode on - Drag the frame around")
			testMode = true
			window.icon:SetTexture("Rift", "Data/UI/item_icons/helm_mouse.dds")
			window.description:SetText("Test")
			window.description:ResizeToText()
			window:SetVisible(true)
			active = false
		else
			print("Testmode off")
			testMode = false
			window:SetVisible(false)
		end
	elseif command == "reset" then
		ProccWatch_Position = {}
		ProccWatch_Skills = {}
		testMode = false
		print("Reset done.")
	else
		print("Possible commands:")
		print("add - Add a new Buff to track")
		print("del - Remove a Buff")
		print("show - See currently tracked buffs")
		print("test - Display a test frame to move around")
	end
end

-- Event handler

table.insert(Event.Buff.Add, {buffAdded, "ProccWatch", "EventAdd"})
table.insert(Event.Buff.Remove, {buffRemoved, "ProccWatch", "Buff Remove"})
table.insert(Event.System.Update.Begin, {update, "ProccWatch", "Update"})
table.insert(Command.Slash.Register("pw"), {slashRegistered, "ProccWatch", "Slash Register"})
table.insert(Event.Addon.SavedVariables.Load.End, {varLoaded, "ProccWatch", "Vars loaded"})
table.insert(Event.Addon.Load.End, {begin, "ProccWatch", "Addon loaded"})

-- Helper

function isPlayer(id)
	if Inspect.Unit.Detail(id).name == Inspect.Unit.Detail("player").name then
		return true
	else
		return false
	end
end

function tContains(t, value)
	for k,v in pairs(t) do
		if(v == value) then
			return k
		end
	end
	
	return nil
end

function plotTable(t)
	print("---- Table ",t," ----")
	for k, v in pairs(t) do
		print(k,v)
	end
	print("-----------")
end

